@@ -1,4 +1,4 @@
-taxi_data_2019 <- as_duckplyr_df(data.frame(
+taxi_data_2019 <- duckdb_tibble(
   vendor_name = character(0),
   pickup_datetime = as.POSIXct(character(0), tz = "UTC"),
   dropoff_datetime = as.POSIXct(character(0), tz = "UTC"),
@@ -23,11 +23,11 @@ taxi_data_2019 <- as_duckplyr_df(data.frame(
   dropoff_location_id = numeric(0),
   year = character(0),
   month = character(0)
-))
+)
 
-zone_map <- as_duckplyr_df(data.frame(
+zone_map <- duckdb_tibble(
   LocationID = numeric(0),
   Borough = character(0),
   Zone = character(0),
   service_zone = character(0)
-))
+)
