@@ -1,7 +1,7 @@
 test_that("fallback_sitrep() default", {
   withr::local_envvar(c(
     "DUCKPLYR_FALLBACK_COLLECT" = "",
-    "DUCKPLYR_FALLBACK_VERBOSE" = "",
+    "DUCKPLYR_FALLBACK_INFO" = "",
     "DUCKPLYR_FALLBACK_AUTOUPLOAD" = "",
     "DUCKPLYR_FALLBACK_LOG_DIR" = "fallback/log/dir",
     "DUCKPLYR_TELEMETRY_FALLBACK_LOGS" = ""
@@ -15,7 +15,7 @@ test_that("fallback_sitrep() default", {
 test_that("fallback_sitrep() enabled", {
   withr::local_envvar(c(
     "DUCKPLYR_FALLBACK_COLLECT" = "1",
-    "DUCKPLYR_FALLBACK_VERBOSE" = "",
+    "DUCKPLYR_FALLBACK_INFO" = "",
     "DUCKPLYR_FALLBACK_AUTOUPLOAD" = "1",
     "DUCKPLYR_FALLBACK_LOG_DIR" = "fallback/log/dir",
     "DUCKPLYR_TELEMETRY_FALLBACK_LOGS" = "1,2,3"
@@ -29,7 +29,7 @@ test_that("fallback_sitrep() enabled", {
 test_that("fallback_sitrep() enabled silent", {
   withr::local_envvar(c(
     "DUCKPLYR_FALLBACK_COLLECT" = "1",
-    "DUCKPLYR_FALLBACK_VERBOSE" = "FALSE",
+    "DUCKPLYR_FALLBACK_INFO" = "TRUE",
     "DUCKPLYR_FALLBACK_AUTOUPLOAD" = "1",
     "DUCKPLYR_FALLBACK_LOG_DIR" = "fallback/log/dir",
     "DUCKPLYR_TELEMETRY_FALLBACK_LOGS" = "1,2,3"
@@ -53,12 +53,6 @@ test_that("fallback_sitrep() disabled", {
   })
 })
 
-test_that("fallback_nudge()", {
-  expect_snapshot({
-    fallback_nudge("{foo:1, bar:2}")
-  })
-})
-
 test_that("summarize()", {
   withr::local_envvar(c(
     "DUCKPLYR_FALLBACK_COLLECT" = "1",
@@ -67,8 +61,7 @@ test_that("summarize()", {
   ))
 
   expect_snapshot({
-    tibble(a = 1, b = 2, c = 3) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(a = 1, b = 2, c = 3) %>%
       summarize(
         .by = a,
         e = sum(b),
@@ -78,6 +71,8 @@ test_that("summarize()", {
 })
 
 test_that("wday()", {
+  skip_if_not_installed("lubridate")
+
   withr::local_envvar(c(
     "DUCKPLYR_FALLBACK_COLLECT" = "1",
     "DUCKPLYR_FALLBACK_VERBOSE" = "TRUE",
@@ -85,8 +80,7 @@ test_that("wday()", {
   ))
 
   expect_snapshot({
-    tibble(a = as.Date("2024-03-08")) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(a = as.Date("2024-03-08")) %>%
       mutate(
         b = lubridate::wday(a, label = TRUE)
       )
@@ -95,8 +89,7 @@ test_that("wday()", {
   local_options(lubridate.week.start = 1)
 
   expect_snapshot({
-    tibble(a = as.Date("2024-03-08")) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(a = as.Date("2024-03-08")) %>%
       mutate(
         b = lubridate::wday(a)
       )
@@ -111,8 +104,7 @@ test_that("strftime()", {
   ))
 
   expect_snapshot({
-    tibble(a = as.Date("2024-03-08")) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(a = as.Date("2024-03-08")) %>%
       mutate(
         b = strftime(a, format = "%Y-%m-%d", tz = "CET")
       )
@@ -129,8 +121,7 @@ test_that("$", {
   ))
 
   expect_snapshot(error = TRUE, {
-    tibble(a = 1, b = 2) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(a = 1, b = 2) %>%
       mutate(c = .env$x)
   })
 })
@@ -145,8 +136,7 @@ test_that("unknown function", {
   foo <- function(...) 3
 
   expect_snapshot({
-    tibble(a = 1, b = 2) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(a = 1, b = 2) %>%
       mutate(c = foo(a, b))
   })
 })
@@ -160,7 +150,7 @@ test_that("row names", {
 
   expect_snapshot({
     mtcars[1:2, ] %>%
-      as_duckplyr_df() %>%
+      as_duckdb_tibble() %>%
       select(mpg, cyl)
   })
 })
@@ -172,10 +162,8 @@ test_that("named column", {
     "DUCKPLYR_FALLBACK_LOG_DIR" = tempdir()
   ))
 
-  expect_snapshot({
-    tibble(a = c(x = 1)) %>%
-      as_duckplyr_df() %>%
-      select(a)
+  expect_snapshot(error = TRUE, {
+    duckdb_tibble(a = c(x = 1))
   })
 })
 
@@ -186,10 +174,8 @@ test_that("named column", {
     "DUCKPLYR_FALLBACK_LOG_DIR" = tempdir()
   ))
 
-  expect_snapshot({
-    tibble(a = matrix(1:4, ncol = 2)) %>%
-      as_duckplyr_df() %>%
-      select(a)
+  expect_snapshot(error = TRUE, {
+    duckdb_tibble(a = matrix(1:4, ncol = 2))
   })
 })
 
@@ -200,10 +186,8 @@ test_that("list column", {
     "DUCKPLYR_FALLBACK_LOG_DIR" = tempdir()
   ))
 
-  expect_snapshot({
-    tibble(a = 1, b = 2, c = list(3)) %>%
-      as_duckplyr_df() %>%
-      select(a, b)
+  expect_snapshot(error = TRUE, {
+    duckdb_tibble(a = 1, b = 2, c = list(3))
   })
 })
 
@@ -216,8 +200,7 @@ test_that("__row_number", {
   ))
 
   expect_snapshot({
-    tibble(`___row_number` = 1, b = 2:3) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(`___row_number` = 1, b = 2:3) %>%
       arrange(b)
   })
 })
@@ -231,8 +214,80 @@ test_that("rel_try()", {
   ))
 
   expect_snapshot({
-    tibble(a = 1) %>%
-      as_duckplyr_df() %>%
+    duckdb_tibble(a = 1) %>%
       count(a, .drop = FALSE, name = "n")
   })
 })
+
+test_that("fallback_config()", {
+  withr::local_envvar(c(
+    "DUCKPLYR_FALLBACK_COLLECT" = NA_character_,
+    "DUCKPLYR_FALLBACK_INFO" = NA_character_,
+    "DUCKPLYR_FALLBACK_LOGGING" = NA_character_,
+    "DUCKPLYR_FALLBACK_AUTOUPLOAD" = NA_character_,
+    "DUCKPLYR_FALLBACK_LOG_DIR" = NA_character_,
+    "DUCKPLYR_FALLBACK_VERBOSE" = NA_character_
+  ))
+
+  config_path <- "fallback.dcf"
+  local_mocked_bindings(fallback_config_path = function() config_path)
+
+  expect_identical(fallback_config_read(), list())
+
+  fallback_config(info = TRUE, logging = TRUE, autoupload = TRUE, log_dir = "/", verbose = TRUE)
+  expect_snapshot_file(config_path, "fallback.dcf")
+
+  expect_snapshot({
+    "No conflicts"
+    fallback_config_load()
+  })
+
+  expect_snapshot({
+    "Reset and set config"
+    fallback_config(reset_all = TRUE, logging = FALSE)
+  })
+  expect_snapshot_file(config_path, "fallback-2.dcf")
+
+  withr::local_envvar(c(
+    DUCKPLYR_FALLBACK_LOGGING = 1
+  ))
+  expect_snapshot({
+    "Conflicts with environment variable and setting"
+    fallback_config_load()
+  })
+
+  expect_snapshot({
+    "Reset config"
+    fallback_config(reset_all = TRUE)
+  })
+  expect_false(file.exists(config_path))
+
+  expect_snapshot(error = TRUE, {
+    fallback_config(boo = FALSE)
+  })
+})
+
+test_that("fallback_config() failure", {
+  withr::local_envvar(c(
+    "DUCKPLYR_FALLBACK_COLLECT" = NA_character_,
+    "DUCKPLYR_FALLBACK_INFO" = NA_character_,
+    "DUCKPLYR_FALLBACK_AUTOUPLOAD" = NA_character_,
+    "DUCKPLYR_FALLBACK_LOG_DIR" = NA_character_,
+    "DUCKPLYR_FALLBACK_VERBOSE" = NA_character_
+  ))
+
+  config_path <- withr::local_tempfile(lines = "boo")
+  local_mocked_bindings(fallback_config_path = function() config_path)
+
+  writeLines("boo", config_path)
+
+  expect_snapshot({
+    fallback_config_load()
+  })
+
+  expect_false(file.exists(config_path))
+
+  expect_snapshot({
+    fallback_config_load()
+  })
+})
